package com.mysql.jdbc;

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import java.util.zip.DataFormatException;
import java.util.zip.Inflater;

/* JADX INFO: Access modifiers changed from: package-private */
/* loaded from: com_mysql_jdbc.jar:com/mysql/jdbc/CompressedInputStream.class */
public class CompressedInputStream extends InputStream {
    private byte[] buffer;
    private Connection connection;
    private InputStream in;
    private byte[] packetHeaderBuffer = new byte[7];
    private int pos = 0;
    private Inflater inflater = new Inflater();

    public CompressedInputStream(Connection conn, InputStream streamFromServer) {
        this.connection = conn;
        this.in = streamFromServer;
    }

    @Override // java.io.InputStream
    public int available() throws IOException {
        if (this.buffer == null) {
            return this.in.available();
        }
        return (this.buffer.length - this.pos) + this.in.available();
    }

    @Override // java.io.InputStream, java.io.Closeable, java.lang.AutoCloseable
    public void close() throws IOException {
        this.in.close();
        this.buffer = null;
        this.inflater = null;
    }

    private void getNextPacketFromServer() throws IOException {
        byte[] uncompressedData;
        int lengthRead = readFully(this.packetHeaderBuffer, 0, 7);
        if (lengthRead < 7) {
            throw new IOException("Unexpected end of input stream");
        }
        int compressedPacketLength = (this.packetHeaderBuffer[0] & 255) + ((this.packetHeaderBuffer[1] & 255) << 8) + ((this.packetHeaderBuffer[2] & 255) << 16);
        int uncompressedLength = (this.packetHeaderBuffer[4] & 255) + ((this.packetHeaderBuffer[5] & 255) << 8) + ((this.packetHeaderBuffer[6] & 255) << 16);
        if (this.connection.getTraceProtocol()) {
            try {
                this.connection.getLog().logTrace(new StringBuffer().append("Reading compressed packet of length ").append(compressedPacketLength).append(" uncompressed to ").append(uncompressedLength).toString());
            } catch (SQLException sqlEx) {
                throw new IOException(sqlEx.toString());
            }
        }
        if (uncompressedLength > 0) {
            uncompressedData = new byte[uncompressedLength];
            byte[] compressedBuffer = new byte[compressedPacketLength];
            readFully(compressedBuffer, 0, compressedPacketLength);
            try {
                this.inflater.reset();
            } catch (NullPointerException e) {
                this.inflater = new Inflater();
            }
            this.inflater.setInput(compressedBuffer);
            try {
                this.inflater.inflate(uncompressedData);
                this.inflater.end();
            } catch (DataFormatException e2) {
                throw new IOException("Error while uncompressing packet from server.");
            }
        } else {
            if (this.connection.getTraceProtocol()) {
                try {
                    this.connection.getLog().logTrace("Packet didn't meet compression threshold, not uncompressing...");
                } catch (SQLException sqlEx2) {
                    throw new IOException(sqlEx2.toString());
                }
            }
            uncompressedData = new byte[compressedPacketLength];
            readFully(uncompressedData, 0, compressedPacketLength);
        }
        if (this.connection.getTraceProtocol()) {
            try {
                this.connection.getLog().logTrace(new StringBuffer().append("Uncompressed packet: \n").append(StringUtils.dumpAsHex(uncompressedData, compressedPacketLength)).toString());
            } catch (SQLException sqlEx3) {
                throw new IOException(sqlEx3.toString());
            }
        }
        if (this.buffer != null && this.pos < this.buffer.length) {
            if (this.connection.getTraceProtocol()) {
                try {
                    this.connection.getLog().logTrace("Combining remaining packet with new: ");
                } catch (SQLException sqlEx4) {
                    throw new IOException(sqlEx4.toString());
                }
            }
            int remaining = this.buffer.length - this.pos;
            byte[] newBuffer = new byte[remaining + uncompressedData.length];
            int newIndex = 0;
            for (int i = this.pos; i < this.buffer.length; i++) {
                int i2 = newIndex;
                newIndex++;
                newBuffer[i2] = this.buffer[i];
            }
            System.arraycopy(uncompressedData, 0, newBuffer, newIndex, uncompressedData.length);
            uncompressedData = newBuffer;
        }
        this.pos = 0;
        this.buffer = uncompressedData;
    }

    private void getNextPacketIfRequired(int numBytes) throws IOException {
        if (this.buffer == null || this.pos + numBytes > this.buffer.length) {
            getNextPacketFromServer();
        }
    }

    @Override // java.io.InputStream
    public int read() throws IOException {
        try {
            getNextPacketIfRequired(1);
            byte[] bArr = this.buffer;
            int i = this.pos;
            this.pos = i + 1;
            return bArr[i] & 255;
        } catch (IOException e) {
            return -1;
        }
    }

    @Override // java.io.InputStream
    public int read(byte[] b) throws IOException {
        return read(b, 0, b.length);
    }

    @Override // java.io.InputStream
    public int read(byte[] b, int off, int len) throws IOException {
        if (b == null) {
            throw new NullPointerException();
        }
        if (off < 0 || off > b.length || len < 0 || off + len > b.length || off + len < 0) {
            throw new IndexOutOfBoundsException();
        }
        if (len <= 0) {
            return 0;
        }
        try {
            getNextPacketIfRequired(len);
            System.arraycopy(this.buffer, this.pos, b, off, len);
            this.pos += len;
            return len;
        } catch (IOException e) {
            return -1;
        }
    }

    private final int readFully(byte[] b, int off, int len) throws IOException {
        if (len < 0) {
            throw new IndexOutOfBoundsException();
        }
        int i = 0;
        while (true) {
            int n = i;
            if (n < len) {
                int count = this.in.read(b, off + n, len - n);
                if (count < 0) {
                    throw new EOFException();
                }
                i = n + count;
            } else {
                return n;
            }
        }
    }

    @Override // java.io.InputStream
    public long skip(long n) throws IOException {
        long count = 0;
        long j = 0;
        while (true) {
            long i = j;
            if (i >= n) {
                break;
            }
            int bytesRead = read();
            if (bytesRead == -1) {
                break;
            }
            count++;
            j = i + 1;
        }
        return count;
    }
}
